; Source Comments 
;
; This is version 1.1 of this sourcecode by Frank Ludvigsen
;
; Put in Filmnet code 0B and TV1000 code 0D for practice. Remember to
; calculate a new Channel ID. Logg with RealMac, Voyager or Maccess, and
; take byte 2 in the A4 command and Xor it with byte 3 in the A4 command and
; add byte 4 in the 88 command (real key index), and you have a new ID.
;
; 15 channel D2-MAC source, deassembled from intel hex file by Frank Ludvigsen
; Original file downloaded from Spinx, www.globalxs.nl/home/s/stegen/index.html
; File from 3.1.96
;
; If you have any comments or questions you can mail me at:
; fkl@sn.no
; or
; Call SatCrypt BBS in Norway :
; +47 64959406
;
; How to compile with MPASM v1.21 : MPASM 15CHMAC7.ASM (:-))
;
; Some comments to help you understand the code:
;
; All values are in hex.
;
; The 5 byte command from the decoder:
;  - Byte 1 : Class (Not used)                   (1c)
;  - Byte 2 : CMD   (Tells us what to do)        (1d)
;  - Byte 3 : P1    (Sub CMD in some cases)      (1e)
;  - Byte 4 : P2    (data, channel ID etc.)      (1f)
;  - Byte 5 : P3    (length of forthcoming data) (20)
;
; Areas used in data memory for special purposes:
;  - 0c - 13 : Hold DES decoded data (8 bytes) part 1
;  - 14 - 1b : Hold DES decoded data (8 bytes) part 2
;  - 18 - 1f : Hold DES encoded data (first part 1, and then part 2)
;  - 1c - 20 : Hold received command  (5 bytes)
;  - 21 - 27 : Hold selected key (7 bytes)
;  - 28 - 2f : Used in the decryption process (Permutated Key)
;  - 20      : #P3 byte receive/send counter
;  - 28      : Number of bits to send
;  - 29      : Hold delay byte
;  - 2a      : Hold byte received or byte to be sent
;  - 2c      : Hold number of bytes to be received
;  - 2d      : Channel ID offset counter
;  - 2e      : Hold #P1 copy
;  - 2f      : Command offset counter
;
; (xx) - The number in bracets is an address, and the byte in the address
;        is used
;
; (w) - Data to be computed is found in W register
;
; (FSR) - Data is moved to the address found in FSR through INDIR
;
; (EEADR) - Contains the address EEDATA is stored in
;
; w=rslt - Result of operation is in W register
; f=rslt - result of operation is in memory address


                TITLE           "15d2mac7.ASM"
                LIST            P=16C84


; Equate Variables 

                INDIR   EQU     00H     ; Indirect pointer, uses FSR address
                RTCC    EQU     01H     ; Real time clock / counter
                PCL     EQU     02H     ; Program counter low byte (8 bits)
                STATUS  EQU     03H     ; Status register
                CY      EQU     0       ; Carry/borrow bit           Status
                DC      EQU     1       ; Digit carry                flag
                Z       EQU     2       ; Zero flag                  byte
                PD      EQU     3       ; Power down                
                T0      EQU     4       ; Time-out bit              
                RP0     EQU     5       ; Register page select bits 
                RP1     EQU     6       ; Not used in 16C84         
                RP2     EQU     7       ; Not used in 16C84         
                FSR     EQU     04H     ; Indirect data memory address pointer
                PORTA   EQU     05H     ; PORT A
                PORTB   EQU     06H     ; PORT B
                EEDATA  EQU     08H     ; Eeprom data
                EEADR   EQU     09H     ; Eeprom address
                PCLATH  EQU     0AH     ; Program counter high byte (5 bits)
                INTCON  EQU     0BH     ; Interrupt control
                TRISA   EQU     085H    ; Data direction port A
                TRISB   EQU     086H    ; Data direction port B
                EECON1  EQU     088H    ; Eeprom control
                W       EQU     0       ; W reg. as destination (register)
                F       EQU     1       ; F file as destination (memory)


                ORG     0000H           ; Start at address 0 in program memory


; Program Entry Point 

                CALL    ConstantDelay   ; Wait a constant period
                CLRF    PCLATH          ; Clear upper 5 bits of programcounter
                GOTO    AnswerToReset   ; Send AnswerToReset string
GotoWOffset     MOVWF   PCL             ; Move W register into lower 8 bits of
                                        ; programcounter, goto (w)


; Answer To Reset String 

                RETLW   03FH            ; AnswerToReset string
                RETLW   067H            ;
                RETLW   02FH            ;
                RETLW   000H            ;
                RETLW   011H            ;
                RETLW   014H            ;
                RETLW   000H            ;
                RETLW   003H            ;
                RETLW   068H            ;


; Answer To Command 04 

                RETLW   000H            ; Answer to Command_04
                RETLW   015H            ;
                RETLW   000H            ;
                RETLW   004H            ;

                RETLW   028H            ; used by CMD c0 - part 1 1/2
                RETLW   00FH            ; used by CMD c0 - part 1 2/2
                RETLW   004H            ; used by CMD c0 - part 2 1/2


; Fixed data 

                RETLW   00FH            ; used by CMD c0 - part 2 2/2
                RETLW   004H            ; used by CMD c0 - part 3 1/2
                RETLW   01FH            ; used by CMD c0 - part 3 2/2

                RETLW   020H            ; used by CMD c0 - part 4 1/3
                RETLW   000H            ; used by CMD c0 - part 4 2/3
                RETLW   08FH            ; used by CMD c0 - part 4 3/3


; Identify Command Bytes 

                RETLW   002H            ; CMD 02 - Receive #P3 bytes, send EOL
                RETLW   004H            ; CMD 04 - Send CMD_04 answer + EOL
                RETLW   006H            ; CMD 06 - Send EOL
                RETLW   0B8H            ; CMD B8 - Send #P3 bytes + EOL
                RETLW   0A4H            ; CMD A4 - Receive #P3 bytes, send EOL
                RETLW   088H            ; CMD 88 - Receive coded data, 77 dat
                RETLW   0C0H            ; CMD C0 - Send decrypted data + EOL
                RETLW   024H            ; CMD 24 - Receive #P3 bytes + EOL
                RETLW   026H            ; CMD 26 - Reply EOL 6e00 (not 9000)


; Execute Commands 

                GOTO    Command_02      ; Execute CMD 02
                GOTO    Command_04_06   ; Execute CMD 04
                GOTO    Command_04_06   ; Execute CMD 06
                GOTO    Command_b8      ; Execute CMD b8
                GOTO    Command_a4      ; Execute CMD a4
                GOTO    Command_88      ; Execute CMD 88
                GOTO    Command_c0      ; Execute CMD c0
                GOTO    Command_24      ; Execute CMD 24
                MOVLW   06EH            ; Move 6e into W register
                GOTO    SendOtherEOL    ; Execute CMD 26


; Unknown data 

                RETLW   04DH            ; Unknown data
                RETLW   053H            ;


; Identify Channel ID bytes 

                                        ; Channel ID's, how to calculate:
                                        ; ID = a4CMDByte2 XOR a4CMDByte3
                                        ;    + 88CMDByte4
                RETLW   03CH            ; CTV channel ID (does this work ES2?)
                RETLW   032H            ; Filmnet channel ID
                RETLW   000H            ; Unused ID
                RETLW   040H            ; BBC Prime channel ID
                RETLW   00EH            ; TV3 channel ID (key 1a)
                RETLW   00FH            ; TV3 channel ID (key 1b)
                RETLW   029H            ; Unknown ID
                RETLW   039H            ; Unknown ID
                RETLW   01FH            ; TV1000 channel ID


; Send AnswerToReset 

AnswerToReset   CLRF    0CH             ; Clear addr. 0c
                MOVLW   009H            ; Move 09 into W register
                MOVWF   20H             ; Move W register into addr. 20
                                        ; 9 bytes to send
                MOVLW   004H            ; Move 04 into W register
                                        ; data offset 04 (answer to reset)
                CALL    SendXBytes      ; Send (20) bytes from W offset


; Send EndOfLine 

SendEOL         MOVLW   090H            ; Move 90 into W register
SendOtherEOL    CALL    SendOneByte     ; Send one byte (w)
                MOVLW   000H            ; Move 00 into W register
                CALL    SendOneByte     ; Send one byte (w)


; Get Command & Interpret 

                MOVF    1EH,W           ; Move byte in addr. 1e into W register
                MOVWF   2EH             ; Move W register into addr. 2e
                                        ; copy data (P1/ID) in 1e to 2e
UseOldP1        CLRF    PCLATH          ; Clear upper 5 bits of programcounter
                MOVLW   005H            ; Move 05 into W register
                MOVWF   2CH             ; Move W register into addr. 2c
                                        ; receive 5 bytes (receiver commando)
                MOVLW   01CH            ; Move 1c into W register
                MOVWF   FSR             ; Move W register into FSR
                                        ; set indirect address to store data
ReceiveCMDLoop  CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                MOVWF   INDIR           ; Move W register into (FSR) addr.
                INCF    FSR,F           ; Increment FSR, Indirect addr. pointer
                DECFSZ  2CH,F           ; Decrement addr. 2c, if 0 skip next
                                        ; decrement receivebyte counter
                GOTO    ReceiveCMDLoop  ; Continue loop, receive more bytes

                MOVLW   009H            ; Move 09 into W register
                MOVWF   2FH             ; Move W register into addr. 2f
CHKForCMDLoop   MOVF    2FH,W           ; Move byte in addr. 2f into W reg.
                ADDLW   019H            ; Add 19 to W register w=22
                                        ; data offset 22 (commands)
                CALL    GotoWOffset     ; Call routine to set W register
                                        ; get commando to compare
                XORWF   1DH,W           ; XOR W reg. with addr. 1d w=rslt
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                GOTO    FoundCMD        ; Recognized a command
                DECFSZ  2FH,F           ; Decrement addr. 2f, if 0 skip next
                GOTO    CHKForCMDLoop   ; Continue loop, check what command
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD

FoundCMD        MOVLW   088H            ; Move 88 into W register
                XORWF   1DH,W           ; XOR W reg. with addr. 1dh, w=rslt
                                        ; check for command 88
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                GOTO    DoNotSendCMD    ; Skip send command echo if command 88
                MOVF    1DH,W           ; Move byte in addr. 1d into W reg.
                                        ; move commando into W register
                CALL    SendOneByte     ; Send one byte (w), echo commando
DoNotSendCMD    MOVF    2FH,W           ; Move byte in addr. 2f into W reg.
                                        ; move whatcommando offset into W reg.
                ADDLW   022H            ; Add 22 to W register w=gotocommando
                MOVWF   PCL             ; Move W register into lower 8 bits
                                        ; of programcounter (execute commando)


; Receive One Coded Byte 

Receive88Byte   MOVLW   077H            ; Move 77 into W register
                CALL    SendOneByte     ; Send one byte (w), ready for one byte
                CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                RETURN                  ; Return to caller


; Command 02 

Command_02      BCF     2CH,00          ; Clear bit 0 in addr. 2c
Command_02_Loop CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                XORLW   040H            ; XOR W register with value 40
                                        ; check for value 40
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                BSF     2CH,00          ; Set bit 0 in addr. 2c
                DECFSZ  20H,F           ; Decrement addr. 20, if 0 skip next
                                        ; decrement #P3 (#receivebytes)
                GOTO    Command_02_Loop ; Continue loop, check for value 40
                                        ; receive #P3 bytes
                BTFSS   2CH,00          ; Test bit 0 (2c), if 1 skip next
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD
                GOTO    SendErrorEOL    ; Send ErrorEOL (9008)


; Command 04 / Command 06 

Command_04_06   MOVLW   00DH            ; Move 0d into W register
                                        ; data offset 0d (command 04 answer)
                CALL    SendXBytes      ; Send (20/#P3) bytes
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD


; Command b8 

Command_b8      MOVF    2EH,W           ; Move Byte in addr. 2e into W reg.
                CALL    SendOneByte     ; Send one byte (w)
                DECFSZ  20H,F           ; Decrement addr. 20, if 0 skip next
                                        ; decrement #P3 (#sendbytes)
                GOTO    Command_b8      ; Continue loop, send #P3 bytes
SendErrorEOL    MOVLW   090H            ; Move 90 into W register
                CALL    SendOneByte     ; Send one byte (w)
                MOVLW   008H            ; Move 08 into W register
                CALL    SendOneByte     ; Send one byte (w)
                GOTO    UseOldP1        ; Dont store new #P1 (1e -> 2e)


; Command a4 

Command_a4      MOVF    1EH,W           ; Move byte in addr.1e into W reg.(#P1)
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                                        ; if #P1 = 0, then done
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD

                ANDLW   006H            ; AND 06 with W register, skip bits
                BTFSC   1EH,01          ; Test bit 1 (1e), if 0 skip next
                GOTO    Continue        ; Error condition ??
                BTFSS   1EH,02          ; Test bit 2 (1e), if 1 skip next
                                        ; if bit 1 in #P1 = 1, then done
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD
                CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                MOVWF   RTCC            ; Move W register into RTCC
                                        ; save first channel ID-byte
                CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                XORWF   RTCC,F          ; XOR RTCC with W register
                                        ; XOR first ID-byte with second ID-byte
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD
Continue        INCF    0CH,F           ; Increment byte in addr. 0c
                BTFSC   0CH,02          ; Test bit 2 (0c), if 0 skip next
                GOTO    SendErrorEOL    ; Send ErrorEOL (9008)
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD


; Command 88 

Command_88      MOVLW   01AH            ; Move 1a into W register
                SUBWF   20H,F           ; Subtract byte in addr. 20 with W reg.
                                        ; receive #P3 - 1a bytes
                MOVF    RTCC,W          ; Move RTTC into W register
                                        ; get half calculated channel ID
                ADDWF   1FH,F           ; Add W reg. to byte in addr. 1fh (#P2)
                                        ; #P2 seems like key-number for channel
                                        ; ID = RTCC + #P2
                CLRF    EEADR           ; Clear EEADR
                                        ; clear eeprom address pointer
                CLRF    2DH             ; Clear byte in addr. 2d
                                        ; set channel ID offset = 0
CheckChannelID  MOVF    2DH,W           ; Move byte in addr. 2d into W register
                ADDLW   02FH            ; Add 2f to W register
                                        ; add 2f to channel ID offset
                CALL    GotoWOffset     ; Call routine to set W register
                XORWF   1FH,W           ; XOR W register with byte in addr. 1f
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                                        ; check for channel ID
                GOTO    FoundAChannel   ; Found a channel, continue.....
                INCF    2DH,F           ; Increment byte in addr. 2d
                                        ; next channel ID offset
                MOVLW   007H            ; Add 07 to W register
                ADDWF   EEADR,F         ; Add W reg. to EEADR, set next key
                                        ; offset, Key offset = Key offset + 7
                BTFSS   2DH,03          ; Test bit 3 (2d), if 1 skip next
                                        ; if unknown, test 0f key's, then go on
                GOTO    CheckChannelID  ; Continue loop, check for ID's

FoundAChannel   MOVLW   021H            ; Move 21 into W register
                MOVWF   FSR             ; Move W register into FSR
                                        ; set indirect address to store data
                MOVLW   007H            ; Move 07 into W register
                MOVWF   2DH             ; Move W register into addr. 2d
CopyChannelKey  BSF     STATUS,RP0      ; Select page 1 (80H - FFH)
                BSF     EEDATA,00       ; Set bit 0 in EECON (set read mode)
                BCF     STATUS,RP0      ; Select page 0 (00H - 7FH)
                MOVF    EEDATA,W        ; Move EEDATA into W register
                                        ; get channel key byte
                MOVWF   INDIR           ; Move W register into INDIR (FSR)
                                        ; save channel key byte
                INCF    FSR,F           ; Increment FSR, next byte
                INCF    EEADR,F         ; Increment EEADR, next byte
                DECFSZ  2DH,F           ; Decrement addr. 2d, if 0 skip next
                GOTO    CopyChannelKey  ; Continue loop, copy 7 key bytes

                CLRF    2FH             ; Clear byte in addr. 2f
ReadCodePart1   CALL    Receive88Byte   ; Receive one byte, not used
                DECFSZ  20H,F           ; Decrement addr. 20, if 0 skip next
                                        ; decrement #P3 (#receivebytes)
                GOTO    ReadCodePart1   ; continue loop, skip #P3-1a bytes

                CALL    ReadAndSaveDES  ; Read 8 DES encrypted data and save
                                        ; first part
                CALL    DoDecryptation  ; Do decryptation
                MOVF    14H,W           ; Move byte in addr. 14 into W reg.
                MOVWF   0CH             ; Move W register into addr. 0c
                MOVF    15H,W           ; Move byte in addr. 15 into W reg.
                MOVWF   0DH             ; Move W register into addr. 0d
                MOVF    16H,W           ; Move byte in addr. 16 into W reg.
                MOVWF   0EH             ; Move W register into addr. 0e
                MOVF    17H,W           ; Move byte in addr. 17 into W reg.
                MOVWF   0FH             ; Move W register into addr. 0f
                MOVF    18H,W           ; Move byte in addr. 18 into W reg.
                MOVWF   10H             ; Move W register into addr. 10
                MOVF    19H,W           ; Move byte in addr. 19 into W reg.
                MOVWF   11H             ; Move W register into addr. 11
                MOVF    1AH,W           ; Move byte in addr. 1a into W reg.
                MOVWF   12H             ; Move W register into addr. 12
                MOVF    1BH,W           ; Move byte in addr. 1b into W reg.
                MOVWF   13H             ; Move W register into addr. 13
                                        ; copy 8 DES bytes to other location
                CALL    ReadAndSaveDES  ; Read 8 DES encrypted data and save
                                        ; second part
                CALL    DoDecryptation  ; Do decryptation
                CALL    Receive88Byte   ; Receive one byte, not used
                CALL    Receive88Byte   ; Receive one byte, not used
                MOVLW   089H            ; Move 89 into W register
                CALL    SendOneByte     ; Send one byte (w)
                MOVLW   008H            ; Move 08 into W register
                MOVWF   20H             ; Move W register into addr. 20
                                        ; Receive 8 more bytes, not used 


; Command 24 

Command_24      CALL    ReceiveOneByte  ; Receive one byte, w=rslt
                DECFSZ  20H,F           ; Decrement byte in addr. 20 (#P3)
                GOTO    Command_24      ; Continue loop, receive #P3 bytes
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD


; Command c0 

Command_c0      MOVLW   006H            ; Move 06 into W register
                XORWF   20H,W           ; XOR W register with addr. 20, w=rslt
                                        ; decoder want just 6 data bytes ?
                BTFSC   STATUS,Z        ; Test Zero-flag, if not Z skip next
                GOTO    SendFixedAnswer ; Send fixed answer instead of DES data
                MOVLW   0EAH            ; Move ea into W register
                CALL    SendOneByte     ; Send one byte (w)
                MOVLW   010H            ; Move 10 into W register
                CALL    SendOneByte     ; Send one byte (w)
                                        ; EA, 10 - decrypted data header
                MOVLW   00CH            ; Move 0c into W register
                MOVWF   FSR             ; Move W register into FSR
                                        ; set indirect address to first DES dat
                MOVLW   002H            ; Move 02 into W register
                SUBWF   20H,F           ; Subtract addr. 20 with W reg. (#P3)
                                        ; already sent 2 bytes (EA, 10)
SendDecrypted   MOVF    INDIR,W         ; Move (FSR) into W register
                CALL    SendOneByte     ; Send one byte (w), send DES data
                INCF    FSR,F           ; Increment FSR, indirect addr. pointer
                DECFSZ  20H,F           ; Decrement addr. 20, if 0 skip next
                                        ; decrement #P3 (#sendDESbytes)
                GOTO    SendDecrypted   ; Continue loop, send decrypted data
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD

SendFixedAnswer MOVF    0CH,W           ; Move byte in addr. 0c into W reg.
                ADDWF   PCL,F           ; Add W reg. to PCL, goto + 06
                NOP                     ; No operation......but makes offset
                GOTO    UseOffset3      ; Use 13 as offset for fixed data
                GOTO    UseOffset2      ; Use 15 as offset for fixed data
                MOVLW   011H            ; Move 11 into W register
                GOTO    UseOtherOffset  ; Use 11 as offset for fixed data
UseOffset2      MOVLW   013H            ; Move 13 into W register
                GOTO    UseOtherOffset  ; Use 13 as offset for fixed data
UseOffset3      MOVLW   015H            ; Move 15 into W register
UseOtherOffset  MOVWF   2FH             ; Move W register into addr. 2f
                MOVLW   000H            ; Move 00 into W register
                CALL    SendOneByte     ; Send one byte (w)
                MOVLW   002H            ; Move 02 into W register
                MOVWF   20H             ; Move W register into addr. 20 (#P3)
                                        ; number of bytes to send
                MOVF    2FH,W           ; Move byte in addr. 2f into W reg.
                                        ; set offset to get data
                CALL    SendXBytes      ; Send (20/#P3) bytes
                MOVLW   003H            ; Move 03 into W register
                MOVWF   20H             ; Move W register into addr. 20 (#P3)
                MOVLW   017H            ; Move 17 into W register, set offset
                CALL    SendXBytes      ; Send (20/#P3) bytes, send c0 answ.4
                GOTO    SendEOL         ; Send EndOfLine (9000), ...next CMD


; Send One Byte 

SendOneByte     MOVWF   2AH             ; Move W register into addr. 2a
                COMF    2AH,F           ; Complement byte in 2a
                                        ; all bytes sent or received are
                                        ; inverted
                MOVLW   04FH            ; Move 4f into W register
                CALL    VariableDelay   ; Wait a variable period (w)
                MOVLW   07FH            ; Move 7f into W register
                TRIS    PORTB           ; Define PORTB I/O as output (bit7)
                BCF     PORTB,07        ; Clear bit 7 in PORTB
                CLRF    2BH             ; Clear byte in addr. 2b, parity
                MOVLW   008H            ; Move 08 into W register
                MOVWF   28H             ; Move W register into addr. 28
                                        ; send 8 bits
SendBitsLoop    CALL    ConstantDelay   ; Wait a constant period
                MOVF    2AH,W           ; Move byte in addr. 2a into W reg.
                                        ; byte to send
                XORWF   2BH,F           ; XOR bute in addr. 2b with W reg.
                                        ; calculate parity bit
                BCF     PORTB,07        ; Clear bit 7 in PORTB
                BTFSC   2AH,07          ; Test bit 7 (2a), if 0 skip next
                                        ; check if bit should be 1 insted of 0
                BSF     PORTB,07        ; Set bit 7 in PORTB
                RLF     2AH,F           ; Rotate left addr. 2a through carry
                                        ; prepare next bit to send
                DECFSZ  28H,F           ; Decrement byte in addr. 28, next bit
                GOTO    SendBitsLoop    ; Continue loop, send more bits

                CALL    ConstantDelay   ; Wait a variable period (w)
                COMF    2BH,F           ; Complement byte in addr. 2b
                                        ; invert calculated parity-bit

                BCF     PORTB,07        ; Clear bit 7 in PORTB
                BTFSC   2BH,07          ; Test bit 7 (2b), if 0 skip next
                                        ; check if bit should be 1 insted of 0
                BSF     PORTB,07        ; Set bit 7 in PORTB
                MOVLW   01DH            ; Move 1d into W register
                CALL    VariableDelay   ; Wait a variable period (w)
                MOVLW   0FFH            ; Move ff into W register
                TRIS    PORTB           ; Define PORTB I/O as input (bit7)
                RETLW   000H            ; Return to caller, W reg. = 00


; Receive One Byte 

ReceiveOneByte  BTFSC   PORTB,07        ; Test bit 7 PORTB, if 0 skip next
                GOTO    ReceiveOneByte  ; Continue loop, wait for incoming byte
                MOVLW   02AH            ; Move 2a into W register
                CALL    VariableDelay   ; Wait a varibale period (w)
                MOVLW   009H            ; Move 09 into W register
                MOVWF   28H             ; Move W register into addr. 28
ReceiveBitsLoop BCF     STATUS,CY       ; Clear carry flag
                BTFSC   PORTB,07        ; Test bit 7 PORTB, if 0 skip next
                                        ; if inbit = 1, then set carryflag
                BSF     STATUS,CY       ; Set carry flag
                RLF     2AH,F           ; Rotate left addr. 2a through carry
                                        ; shift in the incoming bits
                CALL    ConstantDelay   ; Wait a constant period
                DECFSZ  28H,F           ; Decrement byte in addr. 28, next bit
                GOTO    ReceiveBitsLoop ; Continue loop, received more bits

                RRF     2AH,F           ; Rotate right addr. 2a through carry
                                        ; skip the parity bit
                MOVLW   046H            ; Move 46 into W register
                CALL    VariableDelay   ; Wait a variable period (w)
                COMF    2AH,W           ; Complement byte in addr. 2a
                                        ; invert the received byte
                RETURN                  ; Return to caller


; Delay Routines 

ConstantDelay   MOVLW   01BH            ; Move 1b into W register
VariableDelay   MOVWF   29H             ; Move W register into addr. 29
DelayLoop       DECFSZ  29H,F           ; Decrement byte in addr. 29
                GOTO    DelayLoop       ; Continue loop, wait some more....
                RETLW   000H            ; Return to caller, w = 00


; Read And Save Coded data 

ReadAndSaveDES  MOVLW   018H            ; Move 18 into W register
                MOVWF   FSR             ; Move W register into FSR
                                        ; set indirect offset to 18
                MOVLW   008H            ; Move 08 into W register
                MOVWF   2CH             ; Move W register into addr. 2c
                                        ; receive 8 bytes
SaveDESLoop     CALL    Receive88Byte   ; Receive 1 byte of coded data, w=rlst
                MOVWF   INDIR           ; Move W register into (FSR)
                INCF    FSR,F           ; Increment FSR, set to receive next
                DECFSZ  2CH,F           ; Decrement addr. 2c, if 0 skip next
                GOTO    SaveDESLoop     ; continue loop, receive more bytes
                RETLW   000H            ; Return to caller, w = 00


; Send (20) Bytes 

SendXBytes      MOVWF   2DH             ; Move W register into addr. 2d
                                        ; offset of data to send (retw xx)
SendBytesLoop   MOVF    2DH,W           ; Move Byte in addr. 2d into W reg.
                CALL    GotoWOffset     ; Call routine to set W register
                CALL    SendOneByte     ; Send one byte (w)
                INCF    2DH,F           ; Increment byte in addr. 2d, next byte
                DECFSZ  20H,F           ; Decrement addr. 20, if 0 skip next
                GOTO    SendBytesLoop   ; Continue loop, send more bytes
                RETURN                  ; Return to caller


; Do The MAMBA...:-) 

DoDecryptation  MOVLW   010H            ; Move 10 into W register
                                        ; 16 key variations, see rol table
                MOVWF   20H             ; Move W register into addr. 20
BigLoop         DECF    20H,F           ; Decrement byte in addr. 20 (#P3)
                BTFSC   20H,07          ; Test bit 7 (20), if 0 skip next
                                        ; do key rol 16 times (10-11 = FF)
                RETLW   000H            ; Return to caller , w = 00


; Rotate Key Left 

                CALL    GotoWOffset2    ; Call routine to set W register
                                        ; get Key rol data (1 or 2 rols)
                MOVWF   EEDATA          ; Move W register into EEDATA (EEADR)
SmallLoop       BCF     STATUS,CY       ; Clear carry flag, no extra bit!
                RLF     27H,F           ; Rotate left byte in addr. 27
                RLF     26H,F           ; Rotate left byte in addr. 26
                RLF     25H,F           ; Rotate left byte in addr. 25
                RLF     24H,F           ; Rotate left byte in addr. 24
                RLF     23H,F           ; Rotate left byte in addr. 23
                RLF     22H,F           ; Rotate left byte in addr. 22
                RLF     21H,F           ; Rotate left byte in addr. 21
                                        ; bit (0 - 55) = bit (carry + 0 - 54)
                BTFSC   24H,04          ; Bit test 4 (24), if 0 skip next
                BSF     27H,00          ; Bit set 0 (27)
                BCF     24H,04          ; Bit clear 4 (24)
                BTFSC   STATUS,CY       ; Skip if no carry
                BSF     24H,04          ; Bit set 4 (24)
                DECFSZ  EEDATA,F        ; Decrement EEDATA
                GOTO    SmallLoop       ; Continue loop, next byte
                                        ; rol key, according to rol table


; Permutate Key 

                                        ; ok, let's resort the bits a little!
                                        ; Permutate key
                                        ; Sorted byte looks like this:
                                        ; 28 = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 2 (22)
                                        ;      4 = bit 7 (23)
                                        ;      3 = bit 5 (22)
                                        ;      2 = bit 0 (23)
                                        ;      1 = bit 7 (21)
                                        ;      0 = bit 3 (21)
                CLRF    28H             ; Clear byte in addr. 28
                BTFSC   22H,02          ; Bit test 2 (22), if 0 skip next
                BSF     28H,05          ; Bit set 5 (28)
                BTFSC   23H,07          ; Bit test 7 (23), if 0 skip next
                BSF     28H,04          ; Bit set 4 (28)
                BTFSC   22H,05          ; Bit test 5 (22), if 0 skip next
                BSF     28H,03          ; Bit set 3 (28)
                BTFSC   23H,00          ; Bit test 0 (23), if 0 skip next
                BSF     28H,02          ; Bit set 2 (28)
                BTFSC   21H,07          ; Bit test 7 (21), if 0 skip next
                BSF     28H,01          ; Bit set 1 (28)
                BTFSC   21H,03          ; Bit test 3 (21), if 0 skip next
                BSF     28H,00          ; Bit set 0 (28)

                                        ; Sorted byte looks like this:
                                        ; 29 = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 5 (21)
                                        ;      4 = bit 4 (24)
                                        ;      3 = bit 1 (22)
                                        ;      2 = bit 2 (21)
                                        ;      1 = bit 3 (23)
                                        ;      0 = bit 6 (22)
                CLRF    29H             ; Clear byte in addr. 29
                BTFSC   21H,05          ; Bit test 5 (21), if 0 skip next
                BSF     29H,05          ; Bit set 5 (29)
                BTFSC   24H,04          ; Bit test 4 (24), if 0 skip next
                BSF     29H,04          ; Bit set 4 (29)
                BTFSC   22H,01          ; Bit test 1 (22), if 0 skip next
                BSF     29H,03          ; Bit set 3 (29)
                BTFSC   21H,02          ; Bit test 2 (21), if 0 skip next
                BSF     29H,02          ; Bit set 2 (29)
                BTFSC   23H,03          ; Bit test 3 (23), if 0 skip next
                BSF     29H,01          ; Bit set 1 (29)
                BTFSC   22H,06          ; Bit test 6 (22), if 0 skip next
                BSF     29H,00          ; Bit set 0 (29)

                                        ; Sorted byte looks like this:
                                        ; 2a = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 1 (23)
                                        ;      4 = bit 5 (23)
                                        ;      3 = bit 4 (22)
                                        ;      2 = bit 4 (21)
                                        ;      1 = bit 6 (24)
                                        ;      0 = bit 0 (21)
                CLRF    2AH             ; Clear byte in addr. 2a
                BTFSC   23H,01          ; Bit test 1 (23), if 0 skip next
                BSF     2AH,05          ; Bit set 5 (2a)
                BTFSC   23H,05          ; Bit test 5 (23), if 0 skip next
                BSF     2AH,04          ; Bit set 4 (2a)
                BTFSC   22H,04          ; Bit test 4 (22), if 0 skip next
                BSF     2AH,03          ; Bit set 3 (2a)
                BTFSC   21H,04          ; Bit test 4 (21), if 0 skip next
                BSF     2AH,02          ; Bit set 2 (2a)
                BTFSC   24H,06          ; Bit test 6 (24), if 0 skip next
                BSF     2AH,01          ; Bit set 1 (2a)
                BTFSC   21H,00          ; Bit test 0 (21), if 0 skip next
                BSF     2AH,00          ; Bit set 0 (2a)

                                        ; Sorted byte looks like this:
                                        ; 2b = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 0 (22)
                                        ;      4 = bit 1 (21)
                                        ;      3 = bit 5 (24)
                                        ;      2 = bit 4 (23)
                                        ;      1 = bit 3 (22)
                                        ;      0 = bit 6 (21)
                CLRF    2BH             ; Clear byte in addr. 2b
                BTFSC   22H,00          ; Bit test 0 (22), if 0 skip next
                BSF     2BH,05          ; Bit set 5 (2b)
                BTFSC   21H,01          ; Bit test 1 (21), if 0 skip next
                BSF     2BH,04          ; Bit set 4 (2b)
                BTFSC   24H,05          ; Bit test 5 (24), if 0 skip next
                BSF     2BH,03          ; Bit set 3 (2b)
                BTFSC   23H,04          ; Bit test 4 (23), if 0 skip next
                BSF     2BH,02          ; Bit set 2 (2b)
                BTFSC   22H,03          ; Bit test 3 (22), if 0 skip next
                BSF     2BH,01          ; Bit set 1 (2b)
                BTFSC   21H,06          ; Bit test 6 (21), if 0 skip next
                BSF     2BH,00          ; Bit set 0 (2b)

                                        ; Sorted byte looks like this:
                                        ; 2c = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 7 (26)
                                        ;      4 = bit 4 (27)
                                        ;      3 = bit 1 (24)
                                        ;      2 = bit 3 (25)
                                        ;      1 = bit 1 (26)
                                        ;      0 = bit 1 (27)
                CLRF    2CH             ; Clear byte in addr. 2c
                BTFSC   26H,07          ; Bit test 7 (26), if 0 skip next
                BSF     2CH,05          ; Bit set 5 (2c)
                BTFSC   27H,04          ; Bit test 4 (27), if 0 skip next
                BSF     2CH,04          ; Bit set 4 (2c)
                BTFSC   24H,01          ; Bit test 1 (24), if 0 skip next
                BSF     2CH,03          ; Bit set 3 (2c)
                BTFSC   25H,03          ; Bit test 3 (25), if 0 skip next
                BSF     2CH,02          ; Bit set 2 (2c)
                BTFSC   26H,01          ; Bit test 1 (26), if 0 skip next
                BSF     2CH,01          ; Bit set 1 (2c)
                BTFSC   27H,01          ; Bit test 1 (27), if 0 skip next
                BSF     2CH,00          ; Bit set 0 (2c)

                                        ; Sorted byte looks like this:
                                        ; 2d = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 2 (24)
                                        ;      4 = bit 0 (25)
                                        ;      3 = bit 5 (27)
                                        ;      2 = bit 3 (26)
                                        ;      1 = bit 7 (25)
                                        ;      0 = bit 0 (26)
                CLRF    2DH             ; Clear byte in addr. 2d
                BTFSC   24H,02          ; Bit test 2 (24), if 0 skip next
                BSF     2DH,05          ; Bit set 5 (2d)
                BTFSC   25H,00          ; Bit test 0 (25), if 0 skip next
                BSF     2DH,04          ; Bit set 4 (2d)
                BTFSC   27H,05          ; Bit test 5 (27), if 0 skip next
                BSF     2DH,03          ; Bit set 3 (2d)
                BTFSC   26H,03          ; Bit test 3 (26), if 0 skip next
                BSF     2DH,02          ; Bit set 2 (2d)
                BTFSC   25H,07          ; Bit test 7 (25), if 0 skip next
                BSF     2DH,01          ; Bit set 1 (2d)
                BTFSC   26H,00          ; Bit test 0 (26), if 0 skip next
                BSF     2DH,00          ; Bit set 0 (2d)

                                        ; Sorted byte looks like this:
                                        ; 2e = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 4 (26)
                                        ;      4 = bit 7 (27)
                                        ;      3 = bit 1 (25)
                                        ;      2 = bit 0 (27)
                                        ;      1 = bit 6 (25)
                                        ;      0 = bit 3 (27)
                CLRF    2EH             ; Clear byte in addr. 2e
                BTFSC   26H,04          ; Bit test 4 (26), if 0 skip next
                BSF     2EH,05          ; Bit set 5 (2e)
                BTFSC   27H,07          ; Bit test 7 (27), if 0 skip next
                BSF     2EH,04          ; Bit set 4 (2e)
                BTFSC   25H,01          ; Bit test 1 (25), if 0 skip next
                BSF     2EH,03          ; Bit set 3 (2e)
                BTFSC   27H,00          ; Bit test 0 (27), if 0 skip next
                BSF     2EH,02          ; Bit set 2 (2e)
                BTFSC   25H,06          ; Bit test 6 (25), if 0 skip next
                BSF     2EH,01          ; Bit set 1 (2e)
                BTFSC   27H,03          ; Bit test 3 (27), if 0 skip next
                BSF     2EH,00          ; Bit set 0 (2e)

                                        ; Sorted byte looks like this:
                                        ; 2f = 7 = 0
                                        ;      6 = 0
                                        ;      5 = bit 2 (26)
                                        ;      4 = bit 6 (26)
                                        ;      3 = bit 6 (27)
                                        ;      2 = bit 4 (25)
                                        ;      1 = bit 3 (24)
                                        ;      0 = bit 0 (24)
                CLRF    2FH             ; Clear byte in addr. 2f
                BTFSC   26H,02          ; Bit test 2 (26), if 0 skip next
                BSF     2FH,05          ; Bit set 5 (2f)
                BTFSC   26H,06          ; Bit test 6 (26), if 0 skip next
                BSF     2FH,04          ; Bit set 4 (2f)
                BTFSC   27H,06          ; Bit test 6 (27), if 0 skip next
                BSF     2FH,03          ; Bit set 3 (2f)
                BTFSC   25H,04          ; Bit test 4 (25), if 0 skip next
                BSF     2FH,02          ; Bit set 2 (2f)
                BTFSC   24H,03          ; Bit test 3 (24), if 0 skip next
                BSF     2FH,01          ; Bit set 1 (2f)
                BTFSC   24H,00          ; Bit test 0 (24), if 0 skip next
                BSF     2FH,00          ; Bit set 0 (2f)


; Permutate 32 Last Bits In DES-Data/Get Sbox data 

                                        ; Clear 14 - 17 (DES 1st half)
                CLRF    14H             ; Clear byte in addr. 14
                CLRF    15H             ; Clear byte in addr. 15
                CLRF    16H             ; Clear byte in addr. 16
                CLRF    17H             ; Clear byte in addr. 17

                BSF     PCLATH,00       ; Bit set 0 in PCLATH
                RRF     1FH,W           ; Rotate right byte in addr. 1f
                RRF     1CH,W           ; Rotate right byte in addr. 1c
                MOVWF   EEDATA          ; Move W register into EEDATA (EEADR)
                RRF     EEDATA,F        ; Rotate right EEDATA
                RRF     EEDATA,W        ; Rotate right EEDATA, w=rslt
                                        ; create 6 first bits of Expansion
                                        ; to XOR with 6 bits from key to
                                        ; get Sbox index

                XORWF   28H,W           ; XOR W register with (28), w=rslt
                                        ; xor key & expansion
                ANDLW   03FH            ; AND W register with 3f, mask W
                                        ; save the 6 xor'ed bits
                CALL    GotoWOffset     ; Call routine to set W register
                                        ; get first 4 Sbox data bits
                                        ; 7 more(4 bit groups) to go (8 in all)

                MOVWF   EEDATA          ; Move W register into EEDATA (EEADR)
                BTFSC   EEDATA,03       ; Bit test 3 EEDATA, if 0 skip next
                BSF     15H,07          ; Bit set 7 (15)
                BTFSC   EEDATA,02       ; Bit test 2 EEDATA, if 0 skip next
                BSF     16H,07          ; Bit set 7 (16)
                BTFSC   EEDATA,01       ; Bit test 1 EEDATA, if 0 skip next
                BSF     16H,01          ; Bit set 1 (16)
                BTFSC   EEDATA,00       ; Bit test 0 EEDATA, if 0 skip next
                BSF     17H,01          ; Bit set 1 (17)

                                        ; Expansion Xor Key 6 bits(2)=Sbox idx
                RLF     1DH,W           ; Rotate left byte in addr. 1d, w=rslt
                RLF     1CH,W           ; Rotate left byte in addr. 1c, w=rslt
                XORWF   29H,W           ; XOR W register with (29), w=rslt
                ANDLW   03FH            ; AND W register with 3f, mask W
                IORLW   040H            ; OR W register with 40
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,03       ; Bit test 3 EEDATA, if 0 skip next
                BSF     15H,03          ; Bit set 3 (15)
                BTFSC   EEDATA,02       ; Bit test 2 EEDATA, if 0 skip next
                BSF     17H,04          ; Bit set 4 (17)
                BTFSC   EEDATA,01       ; Bit test 1 EEDATA, if 0 skip next
                BSF     14H,06          ; Bit set 6 (14)
                BTFSC   EEDATA,00       ; Bit test 0 EEDATA, if 0 skip next
                BSF     16H,06          ; Bit set 6 (16)

                                        ; Expansion Xor Key 6 bits(3)=Sbox idx
                RRF     1CH,W           ; Rotate right byte in addr. 1c, w=rslt
                RRF     1DH,W           ; Rotate right byte in addr. 1d, w=rslt
                MOVWF   EEDATA          ; Move W register into EEDATA (EEADR)
                RRF     EEDATA,F        ; Rotate right EEDATA
                RRF     EEDATA,W        ; Rotate right EEDATA, w=rslt
                XORWF   2AH,W           ; XOR W register with (2a), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                IORLW   080H            ; OR W register with 80
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,03       ; Bit test 3 EEDATA, if 0 skip next
                BSF     16H,00          ; Bit set 0 (16)
                BTFSC   EEDATA,02       ; Bit test 2 EEDATA, if 0 skip next
                BSF     15H,00          ; Bit set 0 (15)
                BTFSC   EEDATA,01       ; Bit test 1 EEDATA, if 0 skip next
                BSF     17H,02          ; Bit set 2 (17)
                BTFSC   EEDATA,00       ; Bit test 0 EEDATA, if 0 skip next
                BSF     14H,02          ; Bit set 2 (14)

                                        ; Expansion Xor Key 6 bits(4)=Sbox idx
                RLF     1EH,W           ; Rotate left byte in addr. 1e, w=rslt
                RLF     1DH,W           ; Rotate left byte in addr. 1d, w=rslt
                XORWF   2BH,W           ; XOR W register with (2b), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                IORLW   0C0H            ; OR W register with c0
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,03       ; Bit test 3 EEDATA, if 0 skip next
                BSF     17H,06          ; Bit set 6 (17)
                BTFSC   EEDATA,02       ; Bit test 2 EEDATA, if 0 skip next
                BSF     16H,04          ; Bit set 4 (16)
                BTFSC   EEDATA,01       ; Bit test 1 EEDATA, if 0 skip next
                BSF     15H,06          ; Bit set 6 (15)
                BTFSC   EEDATA,00       ; Bit test 0 EEDATA, if 0 skip next
                BSF     14H,07          ; Bit set 7 (14)

                                        ; Expansion Xor Key 6 bits(5)=Sbox idx
                RRF     1DH,W           ; Rotate right byte in addr. 1d, w=rslt
                RRF     1EH,W           ; Rotate right byte in addr. 1e, w=rslt
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                RRF     EEDATA,F        ; Rotate right EEDATA
                RRF     EEDATA,W        ; Rotate right EEDATA, w=rslt
                XORWF   2CH,W           ; XOR W register with (2c), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,07       ; Bit test 7 EEDATA, if 0 skip next
                BSF     14H,00          ; Bit set 0 (14)
                BTFSC   EEDATA,06       ; Bit test 6 EEDATA, if 0 skip next
                BSF     15H,02          ; Bit set 2 (15)
                BTFSC   EEDATA,05       ; Bit test 5 EEDATA, if 0 skip next
                BSF     17H,07          ; Bit set 7 (17)
                BTFSC   EEDATA,04       ; Bit test 4 EEDATA, if 0 skip next
                BSF     14H,05          ; Bit set 5 (14)

                                        ; Expansion Xor Key 6 bits(6)=Sbox idx
                RLF     1FH,W           ; Rotate left byte in addr. 1f, w=rslt
                RLF     1EH,W           ; Rotate left byte in addr. 1e, w=rslt
                XORWF   2DH,W           ; XOR W register with (2d), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                IORLW   040H            ; OR W register with 40
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,07       ; Bit test 7 EEDATA, if 0 skip next
                BSF     14H,04          ; Bit set 4 (14)
                BTFSC   EEDATA,06       ; Bit test 6 EEDATA, if 0 skip next
                BSF     17H,03          ; Bit set 3 (17)
                BTFSC   EEDATA,05       ; Bit test 5 EEDATA, if 0 skip next
                BSF     15H,05          ; Bit set 5 (15)
                BTFSC   EEDATA,04       ; Bit test 4 EEDATA, if 0 skip next
                BSF     16H,05          ; Bit set 5 (16)

                                        ; Expansion Xor Key 6 bits(7)=Sbox idx
                RRF     1EH,W           ; Rotate right byte in addr. 1e, w=rslt
                RRF     1FH,W           ; Rotate right byte in addr. 1f, w=rslt
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                RRF     EEDATA,F        ; Rotate right EEDATA
                RRF     EEDATA,W        ; Rotate right EEDATA, w=rslt
                XORWF   2EH,W           ; XOR W register with (2e), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                IORLW   080H            ; OR W register with 80
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,07       ; Bit test 7 EEDATA, if 0 skip next
                BSF     17H,00          ; Bit set 0 (17)
                BTFSC   EEDATA,06       ; Bit test 6 EEDATA, if 0 skip next
                BSF     15H,04          ; Bit set 4 (15)
                BTFSC   EEDATA,05       ; Bit test 5 EEDATA, if 0 skip next
                BSF     16H,02          ; Bit set 2 (16)
                BTFSC   EEDATA,04       ; Bit test 4 EEDATA, if 0 skip next
                BSF     14H,01          ; Bit set 1 (14)

                                        ; Expansion Xor Key 6 bits(8)=Sbox idx
                RLF     1CH,W           ; Rotate left byte in addr. 1c, w=rslt
                RLF     1FH,W           ; Rotate left byte in addr. 1f, w=rslt
                XORWF   2FH,W           ; XOR W register with (2f), w=rslt
                ANDLW   03FH            ; AND W register with 3f, Mask W
                IORLW   0C0H            ; OR W register with c0
                CALL    GotoWOffset     ; Call routine to set W register
                MOVWF   EEDATA          ; Move W register to EEDATA (EEADR)
                BTFSC   EEDATA,07       ; Bit test 7 EEDATA, if 0 skip next
                BSF     14H,03          ; Bit set 3 (14)
                BTFSC   EEDATA,06       ; Bit test 6 EEDATA, if 0 skip next
                BSF     17H,05          ; Bit set 5 (17)
                BTFSC   EEDATA,05       ; Bit test 5 EEDATA, if 0 skip next
                BSF     15H,01          ; Bit set 1 (15)
                BTFSC   EEDATA,04       ; Bit test 4 EEDATA, if 0 skip next
                BSF     16H,03          ; Bit set 3 (17)


; Generate Next Data Block 

                                        ; Xor 32 bit right data with left 32
                                        ; right data is calculated
                MOVF    18H,W           ; Move byte in addr. 18 into W reg.
                XORWF   14H,F           ; XOR byte in addr. 14 with W reg.
                MOVF    19H,W           ; Move byte in addr. 19 into W reg.
                XORWF   15H,F           ; XOR byte in addr. 15 with W reg.
                MOVF    1AH,W           ; Move byte in addr. 1a into W reg.
                XORWF   16H,F           ; XOR byte in addr. 16 with W reg.
                MOVF    1BH,W           ; Move byte in addr. 1b into W reg.
                XORWF   17H,F           ; XOR byte in addr. 17 with W reg.

                                        ; Move left 32 bit of DES data over
                                        ; to the right
                MOVF    1CH,W           ; Move byte in addr. 1c into W reg.
                MOVWF   18H             ; XOR byte in addr. 18 with W reg.
                MOVF    1DH,W           ; Move byte in addr. 1d into W reg.
                MOVWF   19H             ; XOR byte in addr. 19 with W reg.
                MOVF    1EH,W           ; Move byte in addr. 1e into W reg.
                MOVWF   1AH             ; XOR byte in addr. 1a with W reg.
                MOVF    1FH,W           ; Move byte in addr. 1f into W reg.
                MOVWF   1BH             ; XOR byte in addr. 1b with W reg.

                                        ; Move calculated right 32 bit DES
                                        ; over to the right side
                MOVF    14H,W           ; Move byte in addr. 14 into W reg.
                MOVWF   1CH             ; XOR byte in addr. 1c with W reg.
                MOVF    15H,W           ; Move byte in addr. 15 into W reg.
                MOVWF   1DH             ; XOR byte in addr. 1d with W reg.
                MOVF    16H,W           ; Move byte in addr. 16 into W reg.
                MOVWF   1EH             ; XOR byte in addr. 1e with W reg.
                MOVF    17H,W           ; Move byte in addr. 17 into W reg.
                MOVWF   1FH             ; XOR byte in addr. 1f with W reg.
                GOTO    BigLoop         ; Continue loop

                ORG     02ECH           ; Start at address 02ec in prg. memory


; Get Key Rotation Count 

GotoWOffset2    MOVLW   002H            ; Move 02 into W register
                MOVWF   PCLATH          ; Move W register into PCLATH
                                        ; programcounter upper 5 bits
                MOVF    20H,W           ; Move byte in addr. 20 into W reg.
                ADDWF   PCL,F           ; Relative addressing
                                        ; Add byte in addr. 20 to PCL
                                        ; goto PCL + (20)
                                        ; This is the 16 left rotations
                                        ; for the Key
                RETLW   001H            ; Return to caller, w = 01
                RETLW   002H            ; 02
                RETLW   002H            ; 03
                RETLW   002H            ; 04
                RETLW   002H            ; 05
                RETLW   002H            ; 06
                RETLW   002H            ; 07
                RETLW   001H            ; 08
                RETLW   002H            ; 09
                RETLW   002H            ; 0a
                RETLW   002H            ; 0b
                RETLW   002H            ; 0c
                RETLW   002H            ; 0d
                RETLW   002H            ; 0e
                RETLW   001H            ; 0f
                RETLW   001H            ; 10


; Substitute Boxes 

                RETLW   02EH            ; Return to caller, w = 2e
                RETLW   0E0H            ;
                RETLW   0C4H            ;
                RETLW   0BFH            ;
                RETLW   04DH            ;
                RETLW   027H            ;
                RETLW   011H            ;
                RETLW   0C4H            ;
                RETLW   072H            ;
                RETLW   04EH            ;
                RETLW   0AFH            ;
                RETLW   072H            ;
                RETLW   0BBH            ;
                RETLW   0DDH            ;
                RETLW   068H            ;
                RETLW   011H            ;
                RETLW   083H            ;
                RETLW   05AH            ;
                RETLW   05AH            ;
                RETLW   006H            ;
                RETLW   036H            ;
                RETLW   0FCH            ;
                RETLW   0FCH            ;
                RETLW   0ABH            ;
                RETLW   0D5H            ;
                RETLW   039H            ;
                RETLW   009H            ;
                RETLW   095H            ;
                RETLW   0E0H            ;
                RETLW   083H            ;
                RETLW   097H            ;
                RETLW   068H            ;
                RETLW   044H            ;
                RETLW   0BFH            ;
                RETLW   021H            ;
                RETLW   08CH            ;
                RETLW   01EH            ;
                RETLW   0C8H            ;
                RETLW   0B8H            ;
                RETLW   072H            ;
                RETLW   0ADH            ;
                RETLW   014H            ;
                RETLW   0D6H            ;
                RETLW   0E9H            ;
                RETLW   072H            ;
                RETLW   021H            ;
                RETLW   08BH            ;
                RETLW   0D7H            ;
                RETLW   0FFH            ;
                RETLW   065H            ;
                RETLW   09CH            ;
                RETLW   0FBH            ;
                RETLW   0C9H            ;
                RETLW   003H            ;
                RETLW   057H            ;
                RETLW   09EH            ;
                RETLW   063H            ;
                RETLW   0AAH            ;
                RETLW   03AH            ;
                RETLW   040H            ;
                RETLW   005H            ;
                RETLW   056H            ;
                RETLW   0E0H            ;
                RETLW   03DH            ;
                RETLW   0CFH            ;
                RETLW   0A3H            ;
                RETLW   011H            ;
                RETLW   0FDH            ;
                RETLW   0A8H            ;
                RETLW   044H            ;
                RETLW   0FEH            ;
                RETLW   027H            ;
                RETLW   096H            ;
                RETLW   07FH            ;
                RETLW   02BH            ;
                RETLW   0C2H            ;
                RETLW   063H            ;
                RETLW   098H            ;
                RETLW   084H            ;
                RETLW   05EH            ;
                RETLW   009H            ;
                RETLW   06CH            ;
                RETLW   0D7H            ;
                RETLW   010H            ;
                RETLW   032H            ;
                RETLW   0D1H            ;
                RETLW   04DH            ;
                RETLW   0EAH            ;
                RETLW   0ECH            ;
                RETLW   006H            ;
                RETLW   070H            ;
                RETLW   0B9H            ;
                RETLW   055H            ;
                RETLW   03BH            ;
                RETLW   0BAH            ;
                RETLW   085H            ;
                RETLW   090H            ;
                RETLW   04DH            ;
                RETLW   0EEH            ;
                RETLW   038H            ;
                RETLW   0F7H            ;
                RETLW   02AH            ;
                RETLW   05BH            ;
                RETLW   0C1H            ;
                RETLW   02AH            ;
                RETLW   093H            ;
                RETLW   084H            ;
                RETLW   05FH            ;
                RETLW   0CDH            ;
                RETLW   0F4H            ;
                RETLW   031H            ;
                RETLW   0A2H            ;
                RETLW   075H            ;
                RETLW   0BBH            ;
                RETLW   008H            ;
                RETLW   0E6H            ;
                RETLW   04CH            ;
                RETLW   017H            ;
                RETLW   0A6H            ;
                RETLW   07CH            ;
                RETLW   019H            ;
                RETLW   060H            ;
                RETLW   0D3H            ;
                RETLW   005H            ;
                RETLW   0B2H            ;
                RETLW   08EH            ;
                RETLW   06FH            ;
                RETLW   0D9H            ;
                RETLW   04AH            ;
                RETLW   0DDH            ;
                RETLW   0B0H            ;
                RETLW   007H            ;
                RETLW   029H            ;
                RETLW   0B0H            ;
                RETLW   0EEH            ;
                RETLW   079H            ;
                RETLW   0F6H            ;
                RETLW   043H            ;
                RETLW   003H            ;
                RETLW   094H            ;
                RETLW   08FH            ;
                RETLW   016H            ;
                RETLW   0D5H            ;
                RETLW   0AAH            ;
                RETLW   031H            ;
                RETLW   0E2H            ;
                RETLW   0CDH            ;
                RETLW   038H            ;
                RETLW   09CH            ;
                RETLW   055H            ;
                RETLW   077H            ;
                RETLW   0CEH            ;
                RETLW   05BH            ;
                RETLW   02CH            ;
                RETLW   0A4H            ;
                RETLW   0FBH            ;
                RETLW   062H            ;
                RETLW   08FH            ;
                RETLW   018H            ;
                RETLW   061H            ;
                RETLW   01DH            ;
                RETLW   061H            ;
                RETLW   046H            ;
                RETLW   0BAH            ;
                RETLW   0B4H            ;
                RETLW   0DDH            ;
                RETLW   0D9H            ;
                RETLW   080H            ;
                RETLW   0C8H            ;
                RETLW   016H            ;
                RETLW   03FH            ;
                RETLW   049H            ;
                RETLW   073H            ;
                RETLW   0A8H            ;
                RETLW   0E0H            ;
                RETLW   077H            ;
                RETLW   0ABH            ;
                RETLW   094H            ;
                RETLW   0F1H            ;
                RETLW   05FH            ;
                RETLW   062H            ;
                RETLW   00EH            ;
                RETLW   08CH            ;
                RETLW   0F3H            ;
                RETLW   005H            ;
                RETLW   0EBH            ;
                RETLW   05AH            ;
                RETLW   025H            ;
                RETLW   09EH            ;
                RETLW   032H            ;
                RETLW   027H            ;
                RETLW   0CCH            ;
                RETLW   0D7H            ;
                RETLW   01DH            ;
                RETLW   02DH            ;
                RETLW   0F8H            ;
                RETLW   08EH            ;
                RETLW   0DBH            ;
                RETLW   043H            ;
                RETLW   085H            ;
                RETLW   060H            ;
                RETLW   0A6H            ;
                RETLW   0F6H            ;
                RETLW   03FH            ;
                RETLW   0B9H            ;
                RETLW   070H            ;
                RETLW   01AH            ;
                RETLW   043H            ;
                RETLW   0A1H            ;
                RETLW   0C4H            ;
                RETLW   092H            ;
                RETLW   057H            ;
                RETLW   038H            ;
                RETLW   062H            ;
                RETLW   0E5H            ;
                RETLW   0BCH            ;
                RETLW   05BH            ;
                RETLW   001H            ;
                RETLW   00CH            ;
                RETLW   0EAH            ;
                RETLW   0C4H            ;
                RETLW   09EH            ;
                RETLW   07FH            ;
                RETLW   029H            ;
                RETLW   07AH            ;
                RETLW   023H            ;
                RETLW   0B6H            ;
                RETLW   01FH            ;
                RETLW   049H            ;
                RETLW   0E0H            ;
                RETLW   010H            ;
                RETLW   076H            ;
                RETLW   09CH            ;
                RETLW   04AH            ;
                RETLW   0CBH            ;
                RETLW   0A1H            ;
                RETLW   0E7H            ;
                RETLW   08DH            ;
                RETLW   02DH            ;
                RETLW   0D8H            ;
                RETLW   00FH            ;
                RETLW   0F9H            ;
                RETLW   061H            ;
                RETLW   0C4H            ;
                RETLW   0A3H            ;
                RETLW   095H            ;
                RETLW   0DEH            ;
                RETLW   00BH            ;
                RETLW   0F5H            ;
                RETLW   03CH            ;
                RETLW   032H            ;
                RETLW   057H            ;
                RETLW   058H            ;
                RETLW   062H            ;
                RETLW   084H            ;
                RETLW   0BEH            ;

               ;ORG     2000H           ; You may need this for other
               ;                        ; programmer's than David Tait's or
               ;DATA    3FFF            ; other assembler's than MPASM
               ;DATA    3FFF            ;
               ;DATA    3FFF            ;
               ;DATA    3FFF            ;

               ;ORG     2007H           ; Fuses
               ;                        ;
               ;DATA    3FE1            ;


                ORG     2100H           ; Set EEDATA (EEPROM) address
                                        ; All the key's are in EEDATA area

; CTV Key 

                DATA    0084, 0066, 0030, 00E4, 00DA, 00FA, 0023


; Filmnet Key 

                DATA    00F2, 0058, 0027, 0033, 00E5, 0079, 0061


; Unused Key 

                DATA    0000, 0000, 0000, 0000, 0000, 0000, 0000


; BBC Prime Key 

                DATA    0099, 0001, 0000, 005C, 0063, 00F8, 0050


; TV3 Key (1a) 

                DATA    0018, 0007, 004E, 00FD, 00EF, 00F4, 0050


; TV3 Key (1b) 

                DATA    00A5, 0056, 0032, 0010, 0025, 00D7, 00C8


; Unknown Key 

                DATA    0089, 00C8, 009C, 00A9, 001D, 008D, 0096


; Unknown Key 

                DATA    00BA, 009E, 00DC, 0083, 00C3, 000C, 0089


; TV1000 Key 

                DATA    0090, 0087, 00FD, 00B7, 0085, 0000, 00DF


                DATA    00FF            ; Workbyte in DES

                END                     ; Puh.......
                                        ; Frank Ludvigsen '96

