	title	"EUROCRYPT - Single PIC version - Nov 1995"
	subtitl	"Definitions"
 	list	c=132,P=16C84,R=DEC,F=inhx8M
;-----------------------------------------------------------------------
;			D2MAC.ASM
;-----------------------------------------------------------------------
;	Code for PIC Eurocrypt look-a-like
;-----------------------------------------------------------------------
;
;	Originally from a dis-assembly of commercial pirate card.
;	The assembles with MPALC.EXE.
;
;	Comments/labels by David Parkinson
;
;----------------------------------------------------------------
;	HISTORY
;----------------------------------------------------------------
;  Version Date
;  0.3	18 Nov 1995	DES code restructured to improve
;			efficiency and speed. (>128bytes free)
;
;  0.2	11 Nov 1995	Incorporated fix from Ralph Metzler
;			so the Nokia behaves.
;
;  0.1	Oct/Nov 1995	(Still working on it)
;
;----------------------------------------------------------------

include "16C84.H"		; Standard equates & Macros

;----------------------------------------------------------------
;	Sort this out eventually.  Currently TxByte and
;	RxByte have hard coded instructions for Bit 6.
;	***Check those routines if you change this****

IFbit	equ	6
IFmask	equ	0BFh

;----------------------------------------------------------------
;	Some Macros to help readability

map	Macro	r1,b1,r2,b2	; Map a bit in a byte to another
	btfsc	r1,b1
	bsf	r2,b2
	endm

;	Used in the DES routines to do the key expansion.  This maps
;	specified bits in various bytes into bits 5-0 of a target byte

BuildEkey Macro	r,r1,b1,r2,b2,r3,b3,r4,b4,r5,b5,r6,b6
	clrf	r
	map	r1,b1,r,5
	map	r2,b2,r,4
	map	r3,b3,r,3
	map	r4,b4,r,2
	map	r5,b5,r,1
	map	r6,b6,r,0
	endm

;	Used in the DES routines to store four bits of an S-Box
;	lookup (lo nibble) into the specified byte(s):bit(s)

StoreLo	Macro	r1,b1,r2,b2,r3,b3,r4,b4
	movwf	Etemp
	map	Etemp,3,r1,b1
	map	Etemp,2,r2,b2
	map	Etemp,1,r3,b3
	map	Etemp,0,r4,b4
	endm

;	Used in the DES routines to store four bits of an S-Box
;	lookup (hi nibble) into the specified bytes:bits

StoreHi	Macro	r1,b1,r2,b2,r3,b3,r4,b4
	movwf	Etemp
	map	Etemp,7,r1,b1
	map	Etemp,6,r2,b2
	map	Etemp,5,r3,b3
	map	Etemp,4,r4,b4
	endm

;	Convert data into a 'retlw' type table (8 bytes)

Rdata	Macro	k1,k2,k3,k4,k5,k6,k7,k8
	retlw	k1
	retlw	k2
	retlw	k3
	retlw	k4
	retlw	k5
	retlw	k6
	retlw	k7
	retlw	k8
	endm

;----------------------------------------------------------------
;	File register usage

Identity equ	01
pch	equ	0Ah	; Hi byte of PC counter
entity	equ	0Ch	; Used to hold "entity" type
ECW	equ	0Ch	; Even control word (8 bytes)..
			; ..stored here
r14	equ	14h
r15	equ	15h
r16	equ	16h
r17	equ	17h
r18	equ	18h
r19	equ	19h
r1A	equ	1Ah
r1B	equ	1Bh
r1C	equ	1Ch
r1D	equ	1Dh
r1E	equ	1Eh
r1F	equ	1Fh
e_CLA	equ	1Ch	; Instruction Class
e_INS	equ	1Dh
e_A1	equ	1Eh
e_A2	equ	1Fh
e_LEN	equ	20h

Etemp	equ	EEdata	; Used during 'encrypt' process
Rcount	equ	20h	; Round counter
k1	equ	21h	; +--------+
k2	equ	22h	;          |
k3	equ	23h	;          |
k4	equ	24h	;  DES Key |
k5	equ	25h	;          |
k6	equ	26h	;          |
k7	equ	27h	; +--------+
ek1	equ	28h	; +--------+
ek2	equ	29h	;	   |
ek3	equ	2Ah	; expanded |
ek4	equ	2Bh	; (working)|
ek5	equ	2Ch	;   DES    |
ek6	equ	2Dh	;   key    |
ek7	equ	2Eh	;	   |
ek8	equ	2Fh	; +--------+

Ubcount	equ	28h	; Used as bit counter in UART routine
Dcount	equ	29h	; Used in delay routine when Rx data
Udata	equ	2Ah	; Used to hold UART data
Uparity	equ	2Bh	; Used for parity calculation
RxCount	equ	2Ch	; Used when rx bytes
Icount	equ	2Dh	; Used to determine which Instruction
SavedA1	equ	2Eh	; Save copy of e_A1 instruction parameter

; We use the top bit of the status register to flag if a
; command is not supported (i.e. Absent)

Absent	equ	7
	subtitl	"General Routines"
	page
	org 0
;----------------------------------------------------------------
;	Power up/Reset starting point
;	Wait a bit and then send a 9 byte "answer to reset"

	call	DelayXX
	movlw	9
	movwf	Rcount		; Rcount = 9
	movlw	AnsRes		; Answer to Reset
	call	TxMsg

;----------------------------------------------------------------
;	This is where we send a command completion response
;	It is either 90 00 (all Ok)
;		or   90 08 (Argument Absent)

Complete movlw	90h
	call	TxByte
	swapf	status,w
	andlw	8		; Mask out top bit of status
TxME2	call	TxByte		; send either 0 or 8

;	Do some initialisation and then wait for a 5 byte
;	command from the decoder. This is read into the buffer 1C-20

	movf	e_A1,w		; Save e_A1
	movwf	SavedA1		; (used in "Read Data" command)
	bcf	status,Absent	; Clear bit 7
	clrf	pch		; Ensure Hi address bits are clear
	movlw	5
	movwf	RxCount
	movlw	e_CLA		; Point at r1C (Class byte)
	movwf	ptr
RxCmd	call	RxByte		; Read 5 bytes in to d->20
	movwf	indirect
	incf	ptr,f
	decfsz	RxCount,f
	goto	RxCmd

;	We've received a command, now try and
;	and carry it out.

	movlw	9		; 9 possible instructions
	movwf	Icount
FndCmd	movf	Icount,w	; Index into table
	addlw	InsTab-1	; Table base
	call	LookUp		; Load value
	xorwf	e_INS,w		; Compare with Instruction type
	jz	GotIns		; Skip if the same
	decfsz	Icount,f	; Loop if more
	goto	FndCmd
	goto	Complete

;	We've recognised an instruction...
;	..now act on it.

GotIns	movf	Icount,w
	sublw	6		; Icount >= 6?
	btfss	status,C
	goto	NoAdd		; Yes, get on with it
	movf	e_LEN,f		; No, test length request
	jz	NoAdd		; Skip if no additional data
	movf	e_INS,w		; ..else ACK the instruction
	call	TxByte
NoAdd	movf	Icount,w	; W = Icount
	addlw	Jtable-1	; Add base address
LookUp	movwf	pcl		; Go to it

AnsRes	Rdata	03Fh,067h,02Fh,000h,011h,014h,000h,003h
	retlw	68h

; This table provides a pointer to the "issuer id"

IDtable	Rdata	FNPid,TVid,TVid2,THid,BBCid,FNPid,FNPid,FNPid

; These are the issuer IDs

FNPid	retlw	28h	; FilmNet Plus	00 28 10
	retlw	00

TVid	retlw	04
	retlw	00

TVid2	retlw	04
	retlw	10h

THid	retlw	04
	retlw	30h

BBCid	retlw	28h
	retlw	10h

Msg1	retlw	0x00
	retlw	0x15
	retlw	0x00
	retlw	0x00

Msg5	retlw	0x20
	retlw	0x00
	retlw	0x8f

Msg6	retlw	0x10
	retlw	0x02
	retlw	0xca
	retlw	0x20
	retlw	0xff

;	Table of Instructions we recognise

InsTab	retlw	02h	; Retrieve general purpose data FAC
	retlw	04h	; Request result after selecting FAC
	retlw	0B8h	; Read Data
	retlw	0A4h	; Select issuer entity
	retlw	0C0h	; Read Result
	retlw	006h	; Read block in FAC after selecting block
	retlw	024h	; PIN controlled operations
	retlw	088h	; Compute Control Word
	retlw	026h	; ?????

Jtable	goto	Ins_02
	goto	Ins_04
	goto	Ins_B8
	goto	Ins_A4
	goto	Ins_C0
	goto	Ins_06
	goto	Ins_24
	goto	Ins_88
	goto	Unknown

;----------------------------------------------------------------
; 87 02... (6.15.5)
;	Retrieve general-pupose data FAC
;	3 bytes to come.  Just absorb them and flag 'AA'
;	in final reply if "next block" is requested

Ins_02	bcf	status,Absent
i02_lp	call	RxByte
	xorlw	40h		; Is byte 0x40? (Next block req)
	btfsc	status,z
	bsf	status,Absent	; Flag "AA" response needed if so
	decfsz	Rcount,f
	goto	i02_lp
	goto	Complete

;----------------------------------------------------------------
; 87 04... (6.15.6)
;	Request result after selecting FAC
;	Just send back fixed message 1

Ins_04	movlw	Msg1
	call	TxMsg
	goto	Complete

;----------------------------------------------------------------
; 87 06...(6.15.7)
;	Read block in FAC after selecting block
;	Just send back fixed message 6

Ins_06	movlw	Msg6
	call	TxMsg
	goto	Complete

;----------------------------------------------------------------
;	"Instruction class unknown to card"

Unknown	movlw	06Eh
	call	TxByte
	movlw	0
	goto	TxME2

;----------------------------------------------------------------
; CA B8.... (6.15.4)
;	Read Data - Not too sure what's happening here,
;	(just sending the previous e_A1 byte). 

Ins_B8	movf	SavedA1,w
	call	TxByte
	decfsz	e_LEN,f
	goto	Ins_B8
	bsf	status,Absent
	goto	Complete

;----------------------------------------------------------------
; CA A4.... (6.3.1)
;	Select Issuer entity

Ins_A4	movf	e_A1,w		; Check A1 byte
	jz	A4_iss		; "issuer" if 0
	xorlw	2
	jz	A4_next		; "next" if 2
	xorlw	6		; (reset "2" and check against 4)
	jz	A4_dir		; "direct" if 4
	goto	Complete

A4_iss	clrf	entity		; This is the start
	goto	Complete

A4_next	incf	entity,f	; increment count on "next"
	btfsc	entity,3
	bsf	status,Absent	; Finish when entity=8
	goto	Complete

A4_dir	call	RxByte		; Discard this byte
	call	RxByte		; take this one...
	andlw	0Fh		; ...Mask lo nibble
	movwf	Identity	; Save...
	swapf	Identity,f	; ...in hi nibble
	call	RxByte
	andlw	0F0h
	addwf	Identity,f
	swapf	Identity,f	; Build identity in r01
	goto	Complete

;----------------------------------------------------------------
; CA 88... (6.9)
;	Compute Control Word
;	Work out what key to use and read it from the
;	EEprom data area

Ins_88 	movlw	0x1A
	subwf	e_LEN,f
	movf	Identity,w	; Get "identity"
	andlw	7		; mask to 0-7
	movwf	EEadr		; Set address
	movf	e_A2,w		; identifies the key #
	andlw	1		; Differentiate between two keys...
	addwf	EEadr,f		; (adjust address if necessary)

	movlw	k1		; Point at k1->
	movwf	ptr
	movlw	7
	movwf	Icount		; Set count = 7
I88_rd	bsf	status,5	; switch to page 1
	bsf	EEcon1,RD	; Trigger the "Read" bit
	bcf	status,5	; switch to page 0
	movf	EEdata,w	; Read the data
	movwf	indirect	; Save it
	incf	ptr,f		; Bump addresses (by 9)
	movlw	9
	addwf	EEadr,f
	decfsz	Icount,f	; Loop if more
	goto	I88_rd
I88_clr	call	Tx_77		; Dump any remaining bytes
	call	RxByte
	decfsz	Rcount,f
	goto	I88_clr
	call	Rx8Bytes	; Rx the control word
	call	Encrypt		; Recover seed
	movf	r14,W		; Copy down to 0C-13
	movwf	ECW
	movf	r15,W
	movwf	ECW+1
	movf	r16,W
	movwf	ECW+2
	movf	r17,W
	movwf	ECW+3
	movf	r18,W
	movwf	ECW+4
	movf	r19,W
	movwf	ECW+5
	movf	r1A,W
	movwf	ECW+6
	movf	r1B,W
	movwf	ECW+7
	call	Rx8Bytes	; Rx the next control word
	call	Encrypt		; Recover seed
	call	Tx_77		; Dump the two bytes PI...
	call	RxByte
	call	Tx_77		; ....10
	call	RxByte
	movlw	0x89		; Send 89 ACK
	call	TxByte
	movlw	8
	movwf	e_LEN
Flush	call	RxByte		; Receive & dump 8 bytes of hash
	decfsz	e_LEN,f
	goto	Flush
	goto	Complete

;---------------------------------------------------------
; CA C0 ..... (6.6)
;	Read Result

Ins_C0	movlw	6		; Is requested length==6?
	xorwf	e_LEN,w
	jz	C0_is6

	movlw	0EAh		; No, must be ECW/OCW
	call	TxByte
	movlw	16		; 0x10
	call	TxByte
	movlw	2
	subwf	e_LEN,f		; e_LEN -= 2
	movlw	ECW
	movwf	ptr		; from r0C->
C0_cws	movf	indirect,w
	call	TxByte		; Send seed values
	incf	ptr,1
	decfsz	e_LEN,f
	goto	C0_cws
	goto	Complete

;	Send the IDs for channels for which we have access

C0_is6	movf	entity,w	; get the requested "entity" (0->7)
	addlw	IDtable		; Index into the table
	call	LookUp
	movwf	Icount		; Save pointer
	movlw	0
	call	TxByte		; Send 00
	movlw	2
	movwf	e_LEN		; Set length = 2
	movf	Icount,w
	call	TxMsg		; Send message (e_LEN bytes @ Scount)
	movlw	3
	movwf	e_LEN		; Finish off
	movlw	Msg5
	call	TxMsg
	goto	Complete

;----------------------------------------------------------------
; CA 24... (6.4)
;	PIN code controlled operations
;	Just respond with 25 and absorb all bytes sent(?)

Ins_24	movlw	025h
	call	TxByte
	goto	Flush		; Dump all i/c bytes

	subtitl	"Serial Interface"
	page
;----------------------------------------------------------------
; TxByte - send a byte to the decoder

Tx_77	movlw	77h
TxByte	movwf	Udata		; Put byte in Udata
	comf	Udata,f		; Complemented data is sent
	movlw	32h		; Wait
	call	DelayW
	movlw	IFmask
	tris	PortB		; Set o/p bit
	clrf	Uparity
	bcf	PortB,IFbit	; Clear i/f bit
	movlw	8
	movwf	Ubcount		; Set bit count
Tx_lp	call	DelayXX		; Bit delay
	rrf	Udata,w		; get MSB to bit 6
	xorwf	Uparity,f	; Build parity
	movwf	PortB		; Send next bit
	rlf	Udata,f
	decfsz	Ubcount,f
	goto	Tx_lp
	call	DelayXX
	comf	Uparity,w
	movwf	PortB		; Parity bit
	movlw	0x1D		; Stop bit?
	call	DelayW
	movlw	0FFh		; Port back to i/p
	tris	PortB
	movlw	4Fh		; Final delay
	call	DelayW
	retlw	0

;----------------------------------------------------------------
; RxByte - receive a byte from the decoder

RxByte	btfsc	PortB,IFbit
	goto	RxByte
	movlw	02Ah
	call	DelayW
	movlw	9
	movwf	Ubcount
	bcf	status,c
;RxLoop	movf	PortB,w		; Read in to W (why?)
;	rlf	PortB,f		; Shift MSB to carry
RxLoop	movf	PortB,w		; Read in to W (why?)
	rlf	PortB,w	; Get data into w/7
	addlw	80h	; w/7->carry
	rlf	Udata,f		; Shift carry into 2A
	call	DelayXX
	decfsz	Ubcount,f
	goto	RxLoop
	rrf	Udata,f		; Remove parity bit!
	movlw	0x28
	call	DelayW
	comf	Udata,w		; Correct for data sense
	return

;-----------------------------------------------------------------------
; Delay routines

DelayXX	movlw	27		; 27 loops
DelayW	movwf	Dcount		; Set delay counter
DelayLp	decfsz	Dcount,f
	goto	DelayLp
	retlw	0

;-----------------------------------------------------------------------
; Rx8bytes and store at 18-1F

Rx8Bytes movlw	r18		; point at Data area
	movwf	ptr
	movlw	8		; 8 bytes to receive
	movwf	RxCount
R8_lp	call	Tx_77		; Send a '77'
	call	RxByte
	movwf	indirect
	incf	ptr,f
	decfsz	RxCount,f
	goto	R8_lp
	retlw	0

;----------------------------------------------------------------
; TxMsg
;	Send out a fixed message of length <e_LEN>.
;	Pad with FFs if nec.

TxMsg	movwf	Icount		; Icount = W
	movf	e_LEN,f		; Test e_LEN
	btfsc	status,z	; Is it zero?
	return			; Yes, return....
Txm_lp	movf	Icount,w	; ..else
	call	LookUp
	xorlw	0FFh		; Complement W
	btfss	status,Z	; Skip if it was FF...
	incf	Icount,f	; ...else ++Icount (table index)
	xorlw	0FFh		; Restore W
	call	TxByte		; Send out the byte
	decfsz	e_LEN,f
	goto	Txm_lp
	return

	subtitl	"DES Encryption"
	page
;-----------------------------------------------------------------------
; Encrypt.
;	Using the key in k1-k7 encrypt the data in 18-1F
;	(Working key in r28-2F   - data working area 14-17)

Encrypt	movlw	16		; 16 rounds
	movwf	Rcount
Enc_lp	decf	Rcount,f
	btfsc	Rcount,7	; Exit when Rcount goes -ve
	retlw	0

;	We have 56 bits of key in k1....k7 rotate left the
;	two 28 bits halves (split is in the middle of k4)
;	We do the shift all-at-once and then deal with the
;	carry bits

	call	GetShf		; Get the shift count
	movwf	Etemp
Ishift	bcf	status,c	; Clear carry
	rlf	k7,f		; Shift it all left
	rlf	k6,f
	rlf	k5,f
	rlf	k4,f
	rlf	k3,f		; shift k1-23 left
	rlf	k2,f
	rlf	k1,f
	btfsc	k4,4		; if this is set...
	bsf	k7,0		; ...rotate
	bcf	k4,4		; Clear (in case carry clear)
	btfsc	status,c
	bsf	k4,4		; Rotate carry to k4:4
	decfsz	Etemp,f
	goto	Ishift

;	We've done the shift, now we need to do the Permutation
;	to produce the 48-bit key according to table PC-2.
;	To simplify the following the code we permute the key
;	into the lower 6 bits of 8 bytes. (ek<n> - expanded key n)

	BuildEkey ek1, k2,2,k3,7,k2,5,k3,0,k1,7,k1,3
	BuildEkey ek2, k1,5,k4,4,k2,1,k1,2,k3,3,k2,6
	BuildEkey ek3, k3,1,k3,5,k2,4,k1,4,k4,6,k1,0
	BuildEkey ek4, k2,0,k1,1,k4,5,k3,4,k2,3,k1,6
	BuildEkey ek5, k6,7,k7,4,k4,1,k5,3,k6,1,k7,1
	BuildEkey ek6, k4,2,k5,0,k7,5,k6,3,k5,7,k6,0
	BuildEkey ek7, k6,4,k7,7,k5,1,k7,0,k5,6,k7,3
	BuildEkey ek8, k6,2,k6,6,k7,6,k5,4,k4,3,k4,0

;	We've now built the working key up in ek1-ek8.  Clear
;	the area where we will store the result of the round...

	clrf	r14
	clrf	r15
	clrf	r16
	clrf	r17
	bsf	pch,0		; Set address range to 11xxxxxxxx
				; (was already 10xxxxxxxx for shift lookup)

;	Now select data bits according to the Bit-selection
;	table E and build up six bits in Etemp from data in 1C-1F
;	We want r1F:0 r1C:7 r1C:6 r1C:5 r1C:4 r1C:3

	rrf	r1F,w		; Get LSB of r1F into carry
	rrf	r1C,w		; Get 1C:7-3 into W:6-2
	movwf	Etemp		; Put in Etemp (bottom 2bits are junk)
	rrf	Etemp,f
	rrf	Etemp,w		; W = six bits of data...
	xorwf	ek1,w		; XOR with key
	andlw	03Fh		; Mask to six bits
	call	LookUp		; Do Sbox lookup

;	Finally we have to go through Permutation (P)
;	to get the bits in the right position in the
;	result

	StoreLo	r15,7, r16,7, r16,1, r17,1

;	Now build the next six bits of data in Etemp.
;	We need: r1C:4 r1C:3 r1C:2 r1C:1 r1C:0 r1D:7

	rlf	r1D,w		; Get r1D:7 into carry..
	rlf	r1C,w		; ..and thence into W with other bits
	xorwf	ek2,w		; XOR with key...
	andlw	3Fh		; Mask off six bits
	iorlw	S2_Box & 255	; Next S box
	call	LookUp		; Do Sbox lookup

	StoreLo	r15,3, r17,4, r14,6, r16,6

;	Now build the next six bits of data in Etemp.
;	We need: r1C:0 r1D:7 r1D:6 r1D:5 r1D:4 r1D:3

	rrf	r1C,w		; Put r1C:0 into carry
	rrf	r1D,w		; w=r1C:0,r1D:7.........
	movwf	Etemp
	rrf	Etemp,f
	rrf	Etemp,w		; w=data
	xorwf	ek3,w		; XOR key and data
	andlw	3Fh		; mask out 6 bits
	iorlw	S3_Box & 255
	call	LookUp		; Do S-box lookup

	StoreLo	r16,0, r15,0, r17,2, r14,2

;	Next 6 data bits: r1D:4 r1D:3 r1D:2 r1D:1 r1D:0 r1E:7

	rlf	r1E,w		; Get r1E:7 into carry..
	rlf	r1D,w		; ..and thence into w with the rest
	xorwf	ek4,w		; XOR key and data
	andlw	3Fh		; Mask to six bits
	iorlw	S4_Box & 255
	call	LookUp		; Do s-box lookup

	StoreLo	r17,6, r16,4, r15,6, r14,7

;	Next 6 data bits: r1D:0 r1E:7 r1E:6 r1E:5 r1E:4 r1E:3

	rrf	r1D,w		; Get LSB of r1D into carry
	rrf	r1E,w		; Get 1E:7-3 into W:6-2
	movwf	Etemp		; Put in Etemp (bottom 2bits are junk)
	rrf	Etemp,f
	rrf	Etemp,w		; w=data bits
	xorwf	ek5,w		; XOR with key
	andlw	03Fh		; Mask to 6 bits
	call	LookUp		; Do s-box lookup

	StoreHi	r14,0, r15,2, r17,7, r14,5

;	Next 6 data bits: r1E:4 r1E:3 r1E:2 r1E:1 r1E:0 r1F:7

	rlf	r1F,w		; Get r1F:7 into carry..
	rlf	r1E,w		; ..and thence into w with the rest
	xorwf	ek6,w		; XOR with key
	andlw	3Fh		; mask to 6 bits
	iorlw	S2_Box & 255
	call	LookUp		; Do s-box lookup

	StoreHi	r14,4, r17,3, r15,5, r16,5

;	Next 6 data bits: r1E:0 r1F:7 r1F:6 r1F:5 r1F:4 r1F:3

	rrf	r1E,w		; Put r1E:0 into carry
	rrf	r1F,w		; w=r1E:0,r1F:7.........
	movwf	Etemp
	rrf	Etemp,f
	rrf	Etemp,w
	xorwf	ek7,w		; XOR with key
	andlw	3Fh		; Mask to 6 bits
	iorlw	S3_Box & 255
	call	LookUp		; Do s-box lookup

	StoreHi	r17,0, r15,4, r16,2, r14,1

;	Final 6 data bits: r1F:4 r1F:3 r1F:2 r1F:1 r1F:0 r1C:7

	rlf	r1C,w		; Get r1C:7 into carry..
	rlf	r1F,w		; ..and thence into w (and the rest)
	xorwf	ek8,w		; XOR with key
	andlw	3Fh		; mask to 6 bits
	iorlw	S4_Box & 255
	call	LookUp		; Do s-box lookup

	StoreHi	r14,3, r17,5, r15,1, r16,3

;	We've now done an encryption round, let's
;	XOR with the other half of the data in 18-1B

	movf	r18,w
	xorwf	r14,f
	movf	r19,w
	xorwf	r15,f
	movf	r1A,w
	xorwf	r16,f
	movf	r1B,w
	xorwf	r17,f

;	Reorganise the Left and Right halves
;	1C-1F -> 18-1B,  14-17 -> 1C-1F 

	movf	r1C,w
	movwf	r18
	movf	r1D,w
	movwf	r19
	movf	r1E,w
	movwf	r1A
	movf	r1F,w
	movwf	r1B
	movf	r14,w
	movwf	r1C
	movf	r15,w
	movwf	r1D
	movf	r16,w
	movwf	r1E
	movf	r17,w
	movwf	r1F
	goto	Enc_lp		; Next round

	org	2ECh		; Keep close to S-Box table
;-----------------------------------------------------------------------
; GetShf
;	Look up and return the number of shifts required for
;	this round of the algorithm (table indexed in reverse)

GetShf	movlw	2
	movwf	pch		; Set address range to 10xxxxxxxx
	movf	Rcount,w	; r=Rcount (round count)
	addwf	pcl,f		; index into shift table
	Rdata	1,2,2,2,2,2,2,1
	Rdata	2,2,2,2,2,2,1,1

	list	r=hex
;	These are the S-Box tables.  Each is used twice; each
;	entry is actually 4-bits in size so the first 8 lookups
;	extract the low nibble - the remaining eight the hi nibble.

S1_Box	Rdata	02E,0E0,0C4,0BF,04D,027,011,0C4
	Rdata	072,04E,0AF,072,0BB,0DD,068,011
	Rdata	083,05A,05A,006,036,0FC,0FC,0AB
	Rdata	0D5,039,009,095,0E0,083,097,068
	Rdata	044,0BF,021,08C,01E,0C8,0B8,072
	Rdata	0AD,014,0D6,0E9,072,021,08B,0D7
	Rdata	0FF,065,09C,0FB,0C9,003,057,09E
	Rdata	063,0AA,03A,040,005,056,0E0,03D

S2_Box	Rdata	0CF,0A3,011,0FD,0A8,044,0FE,027
	Rdata	096,07F,02B,0C2,063,098,084,05E
	Rdata	009,06C,0D7,010,032,0D1,04D,0EA
	Rdata	0EC,006,070,0B9,055,03B,0BA,085
	Rdata	090,04D,0EE,038,0F7,02A,05B,0C1
	Rdata	02A,093,084,05F,0CD,0F4,031,0A2
	Rdata	075,0BB,008,0E6,04C,017,0A6,07C
	Rdata	019,060,0D3,005,0B2,08E,06F,0D9

S3_Box	Rdata	04A,0DD,0B0,007,029,0B0,0EE,079
	Rdata	0F6,043,003,094,08F,016,0D5,0AA
	Rdata	031,0E2,0CD,038,09C,055,077,0CE
	Rdata	05B,02C,0A4,0FB,062,08F,018,061
	Rdata	01D,061,046,0BA,0B4,0DD,0D9,080
	Rdata	0C8,016,03F,049,073,0A8,0E0,077
	Rdata	0AB,094,0F1,05F,062,00E,08C,0F3
	Rdata	005,0EB,05A,025,09E,032,027,0CC

S4_Box	Rdata	0D7,01D,02D,0F8,08E,0DB,043,085
	Rdata	060,0A6,0F6,03F,0B9,070,01A,043
	Rdata	0A1,0C4,092,057,038,062,0E5,0BC
	Rdata	05B,001,00C,0EA,0C4,09E,07F,029
	Rdata	07A,023,0B6,01F,049,0E0,010,076
	Rdata	09C,04A,0CB,0A1,0E7,08D,02D,0D8
	Rdata	00F,0F9,061,0C4,0A3,095,0DE,00B
	Rdata	0F5,03C,032,057,058,062,084,0BE

;	This is the current EEdata required for the above
;	org 2100h
;	data 0f2h, 099h, 000h, 000h, 018h, 0abh, 090h, 084h, 000h
;	data 058h, 001h, 000h, 000h, 007h, 0dfh, 087h, 066h, 000h
;	data 027h, 000h, 000h, 000h, 04eh, 0e3h, 0fdh, 030h, 000h
;	data 033h, 05ch, 000h, 000h, 0fdh, 094h, 0b7h, 0e4h, 000h
;	data 0e5h, 063h, 000h, 000h, 0efh, 009h, 085h, 0dah, 000h
;	data 079h, 0f8h, 000h, 000h, 0f4h, 052h, 000h, 0fah, 000h
;	data 061h, 050h, 000h, 000h, 050h, 0dah, 0dfh, 023h, 000h

	END

